// RF24Network - Version: 1.0.8
#include <RF24Network.h>
#include <RF24Network_config.h>
#include <Sync.h>

// RF24 - Version: 1.3.1
#include <RF24.h>
#include "RF24_config.h"
#include <nRF24L01.h>
#include <printf.h>

#include <IRremote.h>
#include "ir_codes.h"
#include <Time.h>
#include <TimeLib.h>
#include <Wire.h>
#include "DS3231.h"

RTClib RTC;

RF24 radio(7, 8);                    // nRF24L01(+) radio attached using Getting Started board
RF24Network network(radio);          // Network uses that radio

const uint16_t this_node = 00;       // Address of our node in Octal format

struct action_to_slave {
  int message;                // 1-> IR code message || 2-> light message
  long IR_code;               // The IR code
  decode_type_t IR_protocol;  // The IR protocol
  int IR_length;              // The IR length*/
};

int time_delay = 1500;

#define lightPin 4    // Light
#define pirPin 2      // PIR

IRsend irsend;
RF24NetworkHeader header;

boolean light_on = false;
const int addr_nano = 8;
boolean bol_activate_presence = true;
boolean bol_activate_movement = true;
boolean bol_test_activated = false;
int device_to_test = -1;
String commI2C = "T000000000000";
time_t last_movement[max_number_of_slave_devices + 1];
long last_check = 0;

struct ps_next_action {
  /* HOUR AND MINUTE NEXT ACTION */
  bool next_action_ir_fixed = false;
  time_t next_action_ir;
  bool next_action_light_fixed = false;
  time_t next_action_light;
};
ps_next_action slave_devices_next_action[max_number_of_slave_devices + 1];

ps_programming ps_program_device[max_number_of_slave_devices+1] = 
{
/************ PRESENCE SIMULATOR PLANNING ************/
  20,23,16,19,1,1,1,1,      // master device (address = 0)
  17,23,7,14,1,1,1,1,        // slave device (address = 1)
  0,0,0,0,0,0,0,0,           // slave device (address = 2)
  0,0,0,0,0,0,0,0,           // slave device (address = 3)
  0,0,0,0,0,0,0,0            // slave device (address = 4)
/************ END PRESENCE SIMULATOR ********************/
};

// IR_PROTOCOLS: UNKNOWN, SONY, NEC, SAMSUNG, RC5, RC6, LG, JVC, WHYNTER, SHARP, DISH, DENON, LEGO_PF

/**************/
// SOME IR_PROTOCOLS AND CODES
// (SAMSUNG,number_of_bits,button POWER,button 1,2,3)
// SAMSUNG,32,0xE0E010EF,0xE0E020DF,0xE0E0609F,0xE0E0A05F

// (SONY,number_of_bits,button POWER,button 1,2,3,4,5,6,7,8,9,0)
// SONY,12,0xA90,0x010,0x810,0x410,0xC10,0x210,0xA10,0x610,0xE10,0x110,0x910
/*************/

st_devices ir_control_codes[max_number_of_slave_devices+1] = 
{ 
  /******************************************/
  /******* IR control codes *****************/
  /******************************************/
  // protocol_id, number_of_bits, 10 IR control codes for the master device (address = 0)
  SAMSUNG,32,0xE0E010EF,0xE0E020DF,0xE0E0609F,0xE0E0A05F,0,0,0,0,0,0,
  
  // protocol_id, number_of_bits, 10 IR control codes for the slave device (address = 1)
  //SAMSUNG,32,0xE0E010EF,0xE0E020DF,0xE0E0609F,0xE0E0A05F,0,0,0,0,0,0,
  NEC,32,0xFF30CF,0xFF18E7,0xFF7A85,0xFF10EF,0xFF38C7,0xFF5AA5,0xFF42BD,0,0,0,
  
  // protocol_id, number_of_bits, 10 IR control codes for the slave device (address = 2)
  UNKNOWN,0,0,0,0,0,0,0,0,0,0,0,
  
  // protocol_id, number_of_bits, 10 IR control codes for the slave device (address = 3)
  UNKNOWN,0,0,0,0,0,0,0,0,0,0,0,
  
   // protocol_id, number_of_bits, 10 IR control codes for the slave device (address = 4)
  UNKNOWN,0,0,0,0,0,0,0,0,0,0,0
  /*********************************************/
  /********* End IR control codes **************/
  /*********************************************/
};

void IR_Transmitt (uint16_t slave_address, int ir_code) {

  Serial.print("IR code: ");
  Serial.println(ir_control_codes[slave_address].control_code[ir_code], HEX);

  for (int i = 0; i < 3; i++) {
    switch (ir_control_codes[slave_address].ir_protocol) {
      case NEC:
        irsend.sendNEC(ir_control_codes[slave_address].control_code[ir_code], ir_control_codes[slave_address].ir_length_code);
        break;
      case SONY:
        irsend.sendSony(ir_control_codes[slave_address].control_code[ir_code], ir_control_codes[slave_address].ir_length_code);
        break;
      case RC5:
        irsend.sendRC5(ir_control_codes[slave_address].control_code[ir_code], ir_control_codes[slave_address].ir_length_code);
        break;
      case RC6:
        irsend.sendRC6(ir_control_codes[slave_address].control_code[ir_code], ir_control_codes[slave_address].ir_length_code);
        break;
      case LG:
        irsend.sendLG(ir_control_codes[slave_address].control_code[ir_code], ir_control_codes[slave_address].ir_length_code);
        break;
      case JVC:
        irsend.sendJVC(ir_control_codes[slave_address].control_code[ir_code], ir_control_codes[slave_address].ir_length_code, true);
        break;
      case WHYNTER:
        irsend.sendWhynter(ir_control_codes[slave_address].control_code[ir_code], ir_control_codes[slave_address].ir_length_code);
        break;
      case SAMSUNG:
        irsend.sendSAMSUNG(ir_control_codes[slave_address].control_code[ir_code], ir_control_codes[slave_address].ir_length_code);
        break;
      case SHARP:
        irsend.sendSharpRaw(ir_control_codes[slave_address].control_code[ir_code], ir_control_codes[slave_address].ir_length_code);
        break;
      case DISH:
        irsend.sendDISH(ir_control_codes[slave_address].control_code[ir_code], ir_control_codes[slave_address].ir_length_code);
        break;
      case DENON:
        irsend.sendDenon(ir_control_codes[slave_address].control_code[ir_code], ir_control_codes[slave_address].ir_length_code);
        break;
      case LEGO_PF:
        irsend.sendLegoPowerFunctions(ir_control_codes[slave_address].control_code[ir_code], ir_control_codes[slave_address].ir_length_code);
        break;
    }
    delay(40);
  }
}

time_t SetDate(int y, int m, int d, int h, int mi, int s  )
{ tmElements_t Date ;
  Date.Second = s;
  Date.Minute = mi;
  Date.Hour = h;
  Date.Day = d ;
  Date.Month = m ;
  Date.Year = y - 1970 ;

  return makeTime(Date);
}

String strDate(time_t  t)
{
  String strD = "";

  if (String(day(t)).length() == 1) strD = strD + "0" + String(day(t));
  else strD = strD + String(day(t));
  if (String(month(t)).length() == 1) strD = strD + "0" + String(month(t));
  else strD = strD + String(month(t));
  strD = strD + String(year(t) - 2000);
  if (String(hour(t)).length() == 1) strD = strD + "0" + String(hour(t));
  else strD = strD + String(hour(t));
  if (String(minute(t)).length() == 1) strD = strD + "0" + String(minute(t));
  else strD = strD + String(minute(t));
  if (String(second(t)).length() == 1) strD = strD + "0" + String(second(t));
  else strD = strD + String(second(t));
  return strD;
}

void printDate(time_t  t)
{
  Serial.print(year(t)) ;    Serial.print(+ "/") ;   Serial.print(month(t));   Serial.print(+ "/") ;
  Serial.print(day(t));    Serial.print( " ") ;
  Serial.print(hour(t));   Serial.print(+ ":") ;   Serial.print(minute(t));   Serial.print(":") ;
  Serial.println(second(t));
}

void setup(void)
{

  radio.begin();
  radio.setPALevel(RF24_PA_MIN);
  radio.setDataRate(RF24_250KBPS);
  network.begin(/*channel*/ 70, /*node address*/ this_node);
  pinMode(lightPin, OUTPUT);
  pinMode(pirPin, INPUT);

  // Set time and date
  Wire.begin(addr_nano);
  DateTime current_date = RTC.now();
  setTime(current_date.hour(), current_date.minute(), current_date.second(),
          current_date.day(), current_date.month(), current_date.year());

  Serial.begin(9600);
  printDate (now());

  Wire.onReceive(receiveEvent);
  Wire.onRequest(requestEvent);

  for (int i = 0; i <= max_number_of_slave_devices; i++) last_movement[i] = 0;
}

void loop() {
  
  if ((bol_test_activated) and (device_to_test >= 0)
      and (device_to_test < max_number_of_slave_devices)) {
  
    unsigned long timenow = millis();
    if (timenow-last_check < 10000) return;
    last_check = millis();

    // Choose randomly a IR code
    Serial.print("Testing device ");
    Serial.println(device_to_test);
    bool ir_code_found = false;
    int ir_code;
    for (int iter = 1; ((iter <= max_number_ir_control_codes * 2) and (not ir_code_found)); iter++) {
      ir_code = random(0, max_number_ir_control_codes);
      if (not (ir_control_codes[device_to_test].control_code[ir_code] == 0)) ir_code_found = true;
    }

    if (device_to_test == this_node) {
      printDate (now());
      // Testing the movement detection
      int pirValue = digitalRead(pirPin);
      if (pirValue == 1) {
        Serial.print("Alarm in master device");
        for (int index = 1; index <= 3; index++) {
          digitalWrite(lightPin, HIGH);
          delay(500);
          digitalWrite(lightPin, LOW);
          delay(500);
        }
        light_on = false;
      }
      // Testing the light
      if (not light_on) {
        digitalWrite(lightPin, HIGH);
        light_on = true;
      } else {
        digitalWrite(lightPin, LOW);
        light_on = false;
      }
      // Testing the IR codes
      if (ir_code_found) {
        IR_Transmitt (device_to_test, ir_code);
      }
      //delay(10000);
      return;
    }

    printDate (now());
    // Testing the movement detection
    boolean light_flashing = false;
    network.update();                               // Check the network regularly for alarms
    while (network.available()) {                 // Is there any incoming data
      uint16_t address_slave;
      network.read(header, &address_slave, sizeof(address_slave));
      if ((address_slave == device_to_test) and (not light_flashing)) {
        light_flashing = true;
        Serial.print("Movement detected in slave device ");
        Serial.println(address_slave);
        for (int index = 1; index <= 3; index++) {
          digitalWrite(lightPin, HIGH);
          delay(500);
          digitalWrite(lightPin, LOW);
          delay(500);
        }
      }
    }

    // Sending a signal to switch on or off the light to the slave
    Serial.print("Sending LIGHT to ");
    Serial.println(device_to_test);
    action_to_slave action_to_slave_t = { 2, 0, UNKNOWN, 0 };
    header.to_node = device_to_test;
    bool message_sent = network.write(header, &action_to_slave_t, sizeof(action_to_slave_t));
    if (message_sent) Serial.println("OK");

    if (ir_code_found) {
      Serial.print("Sending IR ");
      Serial.print(ir_control_codes[device_to_test].control_code[ir_code], HEX);
      Serial.print(" to ");
      Serial.println(device_to_test);
      action_to_slave action_to_slave_t = { 1,
                                            ir_control_codes[device_to_test].control_code[ir_code],
                                            ir_control_codes[device_to_test].ir_protocol,
                                            ir_control_codes[device_to_test].ir_length_code
                                          };
      header.to_node = device_to_test;
      bool message_sent = network.write(header, &action_to_slave_t, sizeof(action_to_slave_t));
      if (message_sent) Serial.println("OK");
    }
    //delay(10000);
    return;
  }

  commI2C = "N000000000000";                      // No movement detection
  network.update();                               // Check the network regularly for alarms
  if (bol_activate_movement) {
    while ( network.available() ) {                 // Is there any incoming data
      uint16_t address_slave;
      network.read(header, &address_slave, sizeof(address_slave));
      if ((address_slave > 0) and (address_slave < max_number_of_slave_devices)) {
        if (now() - last_movement[address_slave] > 10) {
          commI2C = address_slave + strDate(now());   // Movement detection
          last_movement[address_slave] = now();
        }
        Serial.print("Movement detected in slave device ");
        Serial.println(address_slave);
      }
    }

    int pirValue = digitalRead(pirPin);
    if (pirValue == 1) {
      Serial.print("Movement detected in master device");
      if (now() - last_movement[this_node] > 10) {
        commI2C = this_node + strDate(now());   // Movement detection
        last_movement[this_node] = now();
      }
    }
  }

  time_t t = now();
  for (int i = 0; ((i <= max_number_of_slave_devices) and (bol_activate_presence)); i++) {

    if (not (ir_control_codes[i].ir_protocol == UNKNOWN)) {

      time_t current_time = SetDate(year(t), month(t), day(t), hour(t), minute(t), second(t));

      time_t ps_program_device_ini_1 = SetDate(year(t), month(t), day(t), ps_program_device[i].hour_init_ps_1, 0, 0);
      time_t ps_program_device_fin_1 = SetDate(year(t), month(t), day(t), ps_program_device[i].hour_end_ps_1, 0, 0);
      if (ps_program_device[i].hour_end_ps_1 <= ps_program_device[i].hour_init_ps_1)
        time_t ps_program_device_fin_1 = SetDate(year(t), month(t), day(t), ps_program_device[i].hour_end_ps_1, 0, 0) + (24 * 60 * 60 * 1000);

      time_t ps_program_device_ini_2 = SetDate(year(t), month(t), day(t), ps_program_device[i].hour_init_ps_2, 0, 0);
      time_t ps_program_device_fin_2 = SetDate(year(t), month(t), day(t), ps_program_device[i].hour_end_ps_2, 0, 0);
      if (ps_program_device[i].hour_end_ps_2 <= ps_program_device[i].hour_init_ps_2)
        time_t ps_program_device_fin_2 = SetDate(year(t), month(t), day(t), ps_program_device[i].hour_end_ps_2, 0, 0) + (24 * 60 * 60 * 1000);

      /*printDate (current_time);
        printDate(ps_program_device_ini_1);
        printDate(ps_program_device_fin_1);*/

      if (((ps_program_device_ini_1 <= current_time) and (ps_program_device_fin_1 >= current_time)) or
          ((ps_program_device_ini_2 <= current_time) and (ps_program_device_fin_2 >= current_time))) {
        /* action for ir devices */
        if (slave_devices_next_action[i].next_action_ir_fixed) {
          int ir_code;
          if (slave_devices_next_action[i].next_action_ir <= current_time) {
            slave_devices_next_action[i].next_action_ir_fixed = false;

            // Choose randomly a IR code
            bool ir_code_found = false;
            for (int iter = 1; ((iter <= max_number_ir_control_codes * 2) and (not ir_code_found)); iter++) {
              ir_code = random(0, max_number_ir_control_codes);
              if (not (ir_control_codes[i].control_code[ir_code] == 0)) ir_code_found = true;
            }

            if (ir_code_found) {

              commI2C = commI2C + i + strDate(now());
              //Serial.println(commI2C);

              if (i == this_node) IR_Transmitt(this_node, ir_code);
              else {
                // Sending the IR_control_code to the slave
                Serial.print("Sending IR ");
                Serial.print(ir_control_codes[i].control_code[ir_code], HEX);
                Serial.print(" to ");
                Serial.println(i);
                action_to_slave action_to_slave_t = { 1,
                                                      ir_control_codes[i].control_code[ir_code],
                                                      ir_control_codes[i].ir_protocol,
                                                      ir_control_codes[i].ir_length_code
                                                    };
                header.to_node = i;
                bool message_sent = network.write(header, &action_to_slave_t, sizeof(action_to_slave_t));
                if (message_sent) Serial.println("OK");
              }
            }

          }
        } else {
          // Calcular y establecer siguiente action
          slave_devices_next_action[i].next_action_ir_fixed = true;

          slave_devices_next_action[i].next_action_ir = current_time +
              (60 * random(ps_program_device[i].min_minutes_next_action_ir,
                           ps_program_device[i].max_minutes_next_action_ir + 1));
          Serial.print("Next action IR: ");
          printDate (slave_devices_next_action[i].next_action_ir);
          
          if (i == this_node) IR_Transmitt(this_node, 0);
          else {
            // Sending the IR_control_code to the slave
            action_to_slave action_to_slave_t = { 1,
                                                  0,
                                                  ir_control_codes[i].ir_protocol,
                                                  ir_control_codes[i].ir_length_code
                                                };
            header.to_node = i;
            bool message_sent = network.write(header, &action_to_slave_t, sizeof(action_to_slave_t));
          }
          
        }
        /* end action for IR devices */

        /* action for light */
        if (slave_devices_next_action[i].next_action_light_fixed) {

          if (slave_devices_next_action[i].next_action_light <= current_time) {
            slave_devices_next_action[i].next_action_light_fixed = false;

            commI2C = commI2C + i + strDate(now());
            //Serial.println(commI2C);

            if (i == this_node) {
              if (not light_on) {
                digitalWrite(lightPin, HIGH);
                light_on = true;
                Serial.println("LIGHT ON");
              } else {
                digitalWrite(lightPin, LOW);
                light_on = false;
                Serial.println("LIGHT OFF");
              }

            } else {
              // Sending a signal to switch on or off the light to the slave
              Serial.print("Sending LIGHT to ");
              Serial.println(i);
              action_to_slave action_to_slave_t = { 2, 0, UNKNOWN, 0 };
              header.to_node = i;
              bool message_sent = network.write(header, &action_to_slave_t, sizeof(action_to_slave_t));
              if (message_sent) Serial.println("OK");
            }
          }
        } else {
          // Calcular y establecer siguiente action
          slave_devices_next_action[i].next_action_light_fixed = true;
          slave_devices_next_action[i].next_action_light = current_time +
              60 * random(ps_program_device[i].min_minutes_next_action_light,
                          ps_program_device[i].max_minutes_next_action_light + 1);
          Serial.print("Next action light: ");
          printDate (slave_devices_next_action[i].next_action_light);
          if (i == this_node) {
            digitalWrite(lightPin, LOW);
            light_on = false;
          } else {
            action_to_slave action_to_slave_t = { 3, 0, UNKNOWN, 0 };
            header.to_node = i;
            bool message_sent = network.write(header, &action_to_slave_t, sizeof(action_to_slave_t));
          }
        }
        /* end action for light */
      }
    }
  }
  delay(time_delay);
}

// function that executes whenever data is received from master
void receiveEvent_orig(int howMany) {
  int i = 1;
  while (Wire.available()) {
    char c = Wire.read();      /* receive byte as a character */
    //Serial.print(c);
    if (i == 1) {
      if (c == '1') bol_activate_presence = true;
      else bol_activate_presence = false;
    }
    if (i == 2) {
      if (c == '1') bol_activate_movement = true;
      else bol_activate_movement = false;
    }
    if (i == 3) {
      if (c == '1') bol_test_activated = true;
      else bol_test_activated = false;
    }
    if (i == 4) {
      String number = "";
      if (isDigit(c)) {
        number = c;
        device_to_test = number.toInt();
      }
      else device_to_test = -1;
    }
    i++;
  }
  /*Serial.println("-");
    Serial.print(bol_activate_presence);
    Serial.print("-");
    Serial.print(bol_activate_movement);
    Serial.print("-");
    Serial.print(bol_test_activated);
    Serial.print("-");
    Serial.println(device_to_test);*/
}

void receiveEvent(int howMany) {
  String str_received = "";
  while (Wire.available()) {
    char c = Wire.read();      /* receive byte as a character */
    str_received += c;
  }

  Serial.println(str_received);
  
  if (str_received[0] == '1') bol_activate_presence = true;
  else bol_activate_presence = false;
  if (str_received[1] == '1') bol_activate_movement = true;
  else bol_activate_movement = false;
  if (str_received[2] == '1') {
    bol_test_activated = true;
    device_to_test = str_received.substring(3,4).toInt();
  } else {
    bol_test_activated = false;
    device_to_test = -1;
  }
  

  if (str_received[4] == 'P') {
    int dotPosicion = 5;
    String subreceived = "";
    int counter = 1;
    int number;
    int addr_device;

    subreceived = str_received.substring(dotPosicion,str_received.length());
    while (subreceived.length() > 0) {
      if (subreceived.indexOf(",") > 0)
        dotPosicion = subreceived.indexOf(",");
      else
        dotPosicion = subreceived.length();
      String checknumber = subreceived.substring(0,dotPosicion);
      number = checknumber.toInt();
      Serial.println(number);
      
      switch (counter) {
        case 1:   // addr_device
          if (number > max_number_of_slave_devices) return;
          else addr_device = number;
          break;
        case 2:
          ps_program_device[addr_device].hour_init_ps_1 = number;
          break;
        case 3:
          ps_program_device[addr_device].hour_end_ps_1 = number;
          break;
        case 4:
          ps_program_device[addr_device].hour_init_ps_2 = number;
          break;
        case 5:
          ps_program_device[addr_device].hour_end_ps_2 = number;
          break;
        case 6:
          ps_program_device[addr_device].min_minutes_next_action_ir = number;
          break;
        case 7:
          ps_program_device[addr_device].max_minutes_next_action_ir = number;
          break;
        case 8:
          ps_program_device[addr_device].min_minutes_next_action_light = number;
          break;
        case 9:
          ps_program_device[addr_device].max_minutes_next_action_light = number;
          break;
      }
      subreceived = subreceived.substring(dotPosicion+1,str_received.length());
      counter++;
    }
    
  }
  if (str_received[4] == 'I') {
    int dotPosicion = 5;
    String subreceived = "";
    int counter = 1;
    long number;
    int addr_device;
    int index_ircode;

    subreceived = str_received.substring(dotPosicion,str_received.length());
    while (subreceived.length() > 0) {
      if (subreceived.indexOf(",") > 0)
        dotPosicion = subreceived.indexOf(",");
      else
        dotPosicion = subreceived.length();
      String checknumber = subreceived.substring(0,dotPosicion);
      if ((counter == 1) or (counter == 3) or (counter == 4)) number = checknumber.toInt();
      if (counter == 5) {
        char arreglo[10];
        char *pEnd;
        
        checknumber.toCharArray(arreglo,11);
        number = strtoul(arreglo,&pEnd,16);
      }
      Serial.println(number);
      switch (counter) {
        case 1:   // addr_device
          if (number > max_number_of_slave_devices) return;
          else addr_device = number;
          break;
        case 2: // protocol
          if (checknumber == "SONY") ir_control_codes[addr_device].ir_protocol = SONY;
          if (checknumber == "NEC") ir_control_codes[addr_device].ir_protocol = NEC;
          if (checknumber == "RC5") ir_control_codes[addr_device].ir_protocol = RC5;
          if (checknumber == "RC6") ir_control_codes[addr_device].ir_protocol = RC6;
          if (checknumber == "LG") ir_control_codes[addr_device].ir_protocol = LG;
          if (checknumber == "JVC") ir_control_codes[addr_device].ir_protocol = JVC;
          if (checknumber == "WHYNTER") ir_control_codes[addr_device].ir_protocol = WHYNTER;
          if (checknumber == "SAMSUNG") ir_control_codes[addr_device].ir_protocol = SAMSUNG;
          if (checknumber == "DISH") ir_control_codes[addr_device].ir_protocol = DISH;
          if (checknumber == "DENON") ir_control_codes[addr_device].ir_protocol = DENON;
          if (checknumber == "SHARP") ir_control_codes[addr_device].ir_protocol = SHARP;
          if (checknumber == "LEGO_PF") ir_control_codes[addr_device].ir_protocol = LEGO_PF;
          break;
        case 3: // lenght in bits
          ir_control_codes[addr_device].ir_length_code = number;
          break;
        case 4: // index of IR code
          if (((number-1) < max_number_ir_control_codes) and
              (number-1) >= 0) index_ircode = number-1;
          else index_ircode = max_number_ir_control_codes - 1;
          break;
        case 5: // ir control code
          ir_control_codes[addr_device].control_code[index_ircode] = number;
          break;
      }
      subreceived = subreceived.substring(dotPosicion+1,str_received.length());
      counter++;
    }
    /*Serial.println(addr_device);
    Serial.println(ir_control_codes[addr_device].ir_protocol);
    Serial.println(ir_control_codes[addr_device].ir_length_code);
    Serial.println(ir_control_codes[addr_device].control_code[index_ircode]);*/
  }

  //Serial.println("-");
    /*Serial.print(bol_activate_presence);
    Serial.print("-");
    Serial.print(bol_activate_movement);
    Serial.print("-");
    Serial.print(bol_test_activated);
    Serial.print("-");
    Serial.println(device_to_test);*/
}


// function that executes whenever data is requested from master
void requestEvent() {
  char commEnd[27];

  if (commI2C.length() < 25) commI2C = commI2C + "N000000000000";
  commI2C.toCharArray(commEnd, 27);
  Wire.write(commEnd);
}
